@@ -7,6 +7,7 @@ import time
 import uuid
 from abc import ABC, abstractmethod
 
+import pika
 import pulsar
 from confluent_kafka import (
     OFFSET_BEGINNING, Consumer, KafkaError,
@@ -693,3 +694,123 @@ class RedisDestination(DestinationTap):
 
     def close(self):
         self.redis.close()
+
+
+class RabbitMQSource(SourceTap):
+    """ RabbitMQSource reads from RabbitMQ
+
+    >>> from unittest.mock import patch
+    >>> from argparse import ArgumentParser
+    >>> parser = ArgumentParser(conflict_handler='resolve')
+    >>> RabbitMQSource.add_arguments(parser)
+    >>> config = parser.parse_args([])
+    >>> with patch('pika.ConnectionParameters') as c1:
+    ...     with patch('pika.BlockingConnection') as c2:
+    ...         RabbitMQSource(config)
+    RabbitMQSource(queue="in-topic")
+    """
+    kind = 'RABBITMQ'
+
+    def __init__(self, config, logger=logger):
+        super().__init__(config, logger)
+        self.config = config
+        self.topic = config.in_topic
+        self.name = namespacedTopic(config.in_topic, config.namespace)
+        self.timeout = config.timeout
+        self.rabbit = pika.BlockingConnection(pika.ConnectionParameters(config.rabbit))
+        self.channel = self.rabbit.channel()
+        self.channel.queue_declare(queue=self.name)
+        self.delivery_tag = None
+        self.msg = None
+        self.logger.info('RabbitMQSource initialized.')
+
+    def __repr__(self):
+        return 'RabbitMQSource(queue="{}")'.format(
+            self.name,
+        )
+
+    @classmethod
+    def add_arguments(cls, parser):
+        super().add_arguments(parser)
+        parser.add_argument('--rabbit', type=str,
+                            default=os.environ.get('RABBIT', 'localhost'),
+                            help='redis host:port')
+
+    def read(self):
+        timedOut = False
+        lastMessageTime = time.time()
+
+        def callback(ch, method, properties, body):
+            self.delivery_tag = method.delivery_tag
+            self.msg = body
+
+        while not timedOut:
+            try:
+                method, header, body = self.channel.basic_get(self.name)
+                if method:
+                    self.delivery_tag = method.delivery_tag
+                    self.logger.info('Read message %s', self.delivery_tag)
+                    yield self.messageClass(body)
+                    lastMessageTime = time.time()
+            except Exception as ex:
+                self.logger.error(ex)
+                break
+            time.sleep(0.01)
+            if self.timeout > 0 and time.time() - lastMessageTime > self.timeout:
+                self.logger.info('RabbitMQSource timed out.')
+                timedOut = True
+
+    def acknowledge(self):
+        self.logger.info('acknowledged message %s', self.delivery_tag)
+        self.channel.basic_ack(self.delivery_tag)
+
+    def close(self):
+        self.logger.info('RabbitMQSource closed.')
+        self.channel.close()
+        self.rabbit.close()
+
+
+class RabbitMQDestination(DestinationTap):
+    """ RabbitMQDestination writes to RabbitMQ
+
+    >>> from unittest.mock import patch
+    >>> from argparse import ArgumentParser
+    >>> parser = ArgumentParser(conflict_handler='resolve')
+    >>> RabbitMQDestination.add_arguments(parser)
+    >>> config = parser.parse_args([])
+    >>> with patch('pika.ConnectionParameters') as c1:
+    ...     with patch('pika.BlockingConnection') as c2:
+    ...         RabbitMQDestination(config)
+    RabbitMQDestination(queue="out-topic")
+    """
+    kind = 'RABBITMQ'
+
+    def __init__(self, config, logger=logger):
+        super().__init__(config, logger)
+        self.config = config
+        self.topic = config.out_topic
+        self.name = namespacedTopic(config.out_topic, config.namespace)
+        self.rabbit = pika.BlockingConnection(pika.ConnectionParameters(config.rabbit))
+        self.channel = self.rabbit.channel()
+        self.channel.queue_declare(queue=self.name)
+        self.logger.info('RabbitMQDestination initialized.')
+
+    def __repr__(self):
+        return 'RabbitMQDestination(queue="{}")'.format(
+            self.name,
+        )
+
+    @classmethod
+    def add_arguments(cls, parser):
+        super().add_arguments(parser)
+        parser.add_argument('--rabbit', type=str,
+                            default=os.environ.get('RABBIT', 'localhost'),
+                            help='redis host:port')
+
+    def write(self, message):
+        self.channel.basic_publish(exchange='', routing_key=self.name, body=message.serialize())
+
+    def close(self):
+        self.logger.info('RabbitMQDestination closed.')
+        self.channel.close()
+        self.rabbit.close()
